searchNodes=[{"ref":"KitchenSink.html","title":"KitchenSink","type":"module","doc":"Additions to common Elixir modules"},{"ref":"KitchenSink.Algorithms.html","title":"KitchenSink.Algorithms","type":"module","doc":"This is a collection of algorithms. The binary search fit function The search is controlled by the fit function which is passed the value to test and returns :ok if the value is good enough, :high if the next value to test should be higher, or :low if the next value should be lower."},{"ref":"KitchenSink.Algorithms.html#binary_search/5","title":"KitchenSink.Algorithms.binary_search/5","type":"function","doc":""},{"ref":"KitchenSink.Algorithms.html#hybrid_binary_search/4","title":"KitchenSink.Algorithms.hybrid_binary_search/4","type":"function","doc":"binary_search performs a binary search over a range. The binary search function requires a midpoint strategy to be specified. If no strategy is specified, it defaults to :midpoint Examples using the :midpoint strategy iex&gt; names = ~w(Adrian Bill Robert Tony) # Sorted! iex&gt; search_names = fn(position, target) -&gt; ...&gt; current = Enum.at(names, position) ...&gt; cond do ...&gt; current &lt; target -&gt; :high ...&gt; current == target -&gt; :ok ...&gt; current &gt; target -&gt; :low ...&gt; end ...&gt; end iex&gt; iex&gt; Algorithms.binary_search(0, 3, search_names, &quot;Tony&quot;, :midpoint) {:ok, 3} iex&gt; Algorithms.binary_search(0, 3, search_names, &quot;Phil&quot;, :midpoint) {:not_found, 2} It is possible to override the calculation of the midpoint for the binary search, and that is &quot;...left as an exercise for the reader.&quot; It is also possible to binary-search multiple ranges at the same time, in case you are trying to find some balance between of a number of variable factors. Example (albeit a contrived one) of searching multiple ranges simultaneously iex&gt; solve = fn (pos, desired_result) -&gt; ...&gt; result = Enum.reduce(pos, fn (x, acc) -&gt; x + acc end) ...&gt; cond do ...&gt; result &lt; desired_result -&gt; :high ...&gt; result == desired_result -&gt; :ok ...&gt; result &gt; desired_result -&gt; :low ...&gt; end ...&gt; end iex&gt; Algorithms.hybrid_binary_search([2..40, 20..100], solve, 27, :midpoint) Examples using the :interval strategy To see where y = 10 + x³ and y = 1000 + x² intersect iex&gt; solve = fn(position, _) -&gt; ...&gt; y1 = 10 + :math.pow(position, 3) ...&gt; y2 = 1000 + :math.pow(position, 2) ...&gt; difference = y1 - y2 ...&gt; ...&gt; epsilon = 0.0001 ...&gt; ...&gt; cond do ...&gt; abs(difference) &lt; epsilon -&gt; :ok ...&gt; difference &gt; 0.0 -&gt; :low ...&gt; difference &lt; 0.0 -&gt; :high ...&gt; end ...&gt; end iex&gt; iex&gt; {:ok, result} = Algorithms.binary_search(1, 100, solve, 0.0, :interval) iex&gt; Float.round(result, 6) 10.311285"},{"ref":"KitchenSink.Algorithms.html#t:binary_interval_search_fit_func/0","title":"KitchenSink.Algorithms.binary_interval_search_fit_func/0","type":"type","doc":""},{"ref":"KitchenSink.Algorithms.html#t:binary_interval_search_position/0","title":"KitchenSink.Algorithms.binary_interval_search_position/0","type":"type","doc":""},{"ref":"KitchenSink.Algorithms.html#t:binary_interval_search_result/0","title":"KitchenSink.Algorithms.binary_interval_search_result/0","type":"type","doc":""},{"ref":"KitchenSink.Algorithms.html#t:binary_search_fit_func/0","title":"KitchenSink.Algorithms.binary_search_fit_func/0","type":"type","doc":""},{"ref":"KitchenSink.Algorithms.html#t:binary_search_position/0","title":"KitchenSink.Algorithms.binary_search_position/0","type":"type","doc":""},{"ref":"KitchenSink.Algorithms.html#t:binary_search_result/0","title":"KitchenSink.Algorithms.binary_search_result/0","type":"type","doc":""},{"ref":"KitchenSink.Algorithms.html#t:binary_search_strategy/0","title":"KitchenSink.Algorithms.binary_search_strategy/0","type":"type","doc":""},{"ref":"KitchenSink.Boolean.html","title":"KitchenSink.Boolean","type":"module","doc":"this module is for Boolean functions"},{"ref":"KitchenSink.Boolean.html#parse/1","title":"KitchenSink.Boolean.parse/1","type":"function","doc":"Parse boolean either from string or integer, if it&#39;s not a boolean then it returns :error iex&gt; parse(&quot;True&quot;) true iex&gt; parse(&quot;Lie&quot;) :error"},{"ref":"KitchenSink.CSV.html","title":"KitchenSink.CSV","type":"module","doc":"This module is for CSV helper functions."},{"ref":"KitchenSink.CSV.html#fill/2","title":"KitchenSink.CSV.fill/2","type":"function","doc":"transforms a sparse list into a new list of the same size with the sparse values filled in with the previous non-sparse values. Example iex&gt; KitchenSink.CSV.fill([1,&quot;&quot;,&quot;&quot;,2,&quot;&quot;,3,&quot;&quot;], &quot;&quot;) [1,1,1,2,2,3,3]"},{"ref":"KitchenSink.Function.html","title":"KitchenSink.Function","type":"module","doc":"this module is for higher order functions"},{"ref":"KitchenSink.Function.html#compose/1","title":"KitchenSink.Function.compose/1","type":"function","doc":"executes a value over a list of functions in reverse order (this is functional composition without macros)"},{"ref":"KitchenSink.Function.html#identity/1","title":"KitchenSink.Function.identity/1","type":"function","doc":"returns the objects you give it. useful in Enum.flat_map, or Enum.filter"},{"ref":"KitchenSink.Function.html#juxt/1","title":"KitchenSink.Function.juxt/1","type":"function","doc":""},{"ref":"KitchenSink.Function.html#juxt/2","title":"KitchenSink.Function.juxt/2","type":"function","doc":"from clojure. takes in a list of functions, outputs a function that takes in a list of values, then applies the functions to each value. sorta like a super Enum.map"},{"ref":"KitchenSink.Function.html#pipe/1","title":"KitchenSink.Function.pipe/1","type":"function","doc":"executes a value over a list of functions (this is functional composition without macros)"},{"ref":"KitchenSink.Function.html#reverse/1","title":"KitchenSink.Function.reverse/1","type":"function","doc":"reverses the arguments of a function."},{"ref":"KitchenSink.Integer.html","title":"KitchenSink.Integer","type":"module","doc":"this module is for Integer functions"},{"ref":"KitchenSink.Integer.html#ensure_integer/1","title":"KitchenSink.Integer.ensure_integer/1","type":"function","doc":"Make sure that we get an integer either from an integer or string iex&gt; ensure_integer(&quot;1&quot;) 1 iex&gt; ensure_integer(2) 2"},{"ref":"KitchenSink.Keyword.html","title":"KitchenSink.Keyword","type":"module","doc":"This module is a mixin for the elixir keyword namespace"},{"ref":"KitchenSink.Keyword.html#deep_merge/2","title":"KitchenSink.Keyword.deep_merge/2","type":"function","doc":""},{"ref":"KitchenSink.List.html","title":"KitchenSink.List","type":"module","doc":"this module is for List functions"},{"ref":"KitchenSink.List.html#index_by/2","title":"KitchenSink.List.index_by/2","type":"function","doc":"Creates an indexed list from a list of maps using a key_function or a path. If a path is given then the Access module will be used. This function is very similar to group_by, however the values will not be grouped. There should be a 1-to-1 mapping for the key_fun to item in the list. For different mapping ratio duplicate keys will be dropped, according to Map.new/1 logic Examples iex&gt; [ ...&gt; %{name: :a, other: 1}, ...&gt; %{name: :b, other: 4} ...&gt; ] |&gt; index_by(:name) %{ a: %{name: :a, other: 1}, b: %{name: :b, other: 4} }"},{"ref":"KitchenSink.List.html#index_on/3","title":"KitchenSink.List.index_on/3","type":"function","doc":"Takes a list of maps, transforms it into a map of maps with their value being the value_key. Basically making a look-up table. Examples iex&gt; [ ...&gt; %{a: 1, b: &quot;x&quot;}, ...&gt; %{a: 2, b: &quot;y&quot;}, ...&gt; %{a: 3, b: &quot;z&quot;} ...&gt; ] |&gt; index_on([:a], :b) %{ %{a: 1} =&gt; &quot;x&quot;, %{a: 2} =&gt; &quot;y&quot;, %{a: 3} =&gt; &quot;z&quot; } iex&gt; [ ...&gt; %{a: 1, b: &quot;x&quot;}, ...&gt; %{a: 2, b: &quot;y&quot;}, ...&gt; %{a: 3, b: &quot;z&quot;} ...&gt; ] |&gt; index_on(:a, :b) %{ %{a: 1} =&gt; &quot;x&quot;, %{a: 2} =&gt; &quot;y&quot;, %{a: 3} =&gt; &quot;z&quot; }"},{"ref":"KitchenSink.List.html#pluck/2","title":"KitchenSink.List.pluck/2","type":"function","doc":"A convenient version of what is perhaps the most common use-case for map: extracting a list of property values. With a List of Maps, extract 1 value defined by the key you give to pluck. Examples iex&gt; pluck([%{b: 1}, %{a: 2}, %{a: 3}], :a) [nil, 2, 3]"},{"ref":"KitchenSink.Map.html","title":"KitchenSink.Map","type":"module","doc":"This module is a mixin for the elixir map namespace"},{"ref":"KitchenSink.Map.html#clean_struct/1","title":"KitchenSink.Map.clean_struct/1","type":"function","doc":"clean_struct take in a struct and removes the default values from it and returns a map"},{"ref":"KitchenSink.Map.html#deep_merge/3","title":"KitchenSink.Map.deep_merge/3","type":"function","doc":"deep_merge overcomes a limitation of Map.merge in that it will merge trees. deep_merge will attempt to make the minimum possible change when it merges 2 trees."},{"ref":"KitchenSink.Map.html#deep_merge_list/2","title":"KitchenSink.Map.deep_merge_list/2","type":"function","doc":""},{"ref":"KitchenSink.Map.html#diff/2","title":"KitchenSink.Map.diff/2","type":"function","doc":"Compares two maps and returns a list of paths (used in Access) and the differences in values between the maps iex&gt; KitchenSink.Map.diff(%{a: 1, b: 2}, %{a: 3, b: 2}) [{[:a], 1, 3}] iex&gt; KitchenSink.Map.diff(%{c: %{d: 1}}, %{c: %{d: 4}}) [{[:c, :d], 1, 4}] iex&gt; KitchenSink.Map.diff(%{c: %{d: 1, e: 7}}, %{c: %{d: 4}}) [ {[:c, :d], 1, 4}, {[:c, :e], 7, nil} ] iex&gt; KitchenSink.Map.diff(%{c: %{d: 1}}, %{c: %{d: 4, e: 7}}) [ {[:c, :d], 1, 4}, {[:c, :e], nil, 7} ]"},{"ref":"KitchenSink.Map.html#do_trim/2","title":"KitchenSink.Map.do_trim/2","type":"function","doc":""},{"ref":"KitchenSink.Map.html#is_enumerable/1","title":"KitchenSink.Map.is_enumerable/1","type":"function","doc":""},{"ref":"KitchenSink.Map.html#key_paths/1","title":"KitchenSink.Map.key_paths/1","type":"function","doc":"Takes a Map and returns a List of Paths. A Path is a list of keys that you can use to access a leaf node/value in a Map. Useful with get_in Access syntax. get_in(my_map, [:a, :b, :c]) Example iex&gt; KitchenSink.Map.key_paths(%{a: %{b: 1, c: 2}, d: 3}) [[:a, :b], [:a, :c], [:d]]"},{"ref":"KitchenSink.Map.html#make_nested/2","title":"KitchenSink.Map.make_nested/2","type":"function","doc":"takes in a list of keys, and a value. creates a nested map of each successive key nested as a child, with the most nested map having the value value given to the funciton"},{"ref":"KitchenSink.Map.html#merge/1","title":"KitchenSink.Map.merge/1","type":"function","doc":"merge an array of maps! using merge/1 Example: iex&gt; import KitchenSink.Map iex&gt; merge [%{a: 1}, %{b: 2}, %{c: 3}, %{d: 4}] %{a: 1, b: 2, c: 3, d: 4}"},{"ref":"KitchenSink.Map.html#remap_keys/2","title":"KitchenSink.Map.remap_keys/2","type":"function","doc":"Similar to rename_key/3 All of the keys that don&#39;t have a remapping are preserved in the returned Map"},{"ref":"KitchenSink.Map.html#remap_keys/3","title":"KitchenSink.Map.remap_keys/3","type":"function","doc":"Similar to rename_key, however this uses a key_map that is larger. key_map = %{ old_key1 =&gt; new_key1, old_key2 =&gt; new_key2, } If any of the values of the key_map are lists, then the output for that key will be a nested map based on the items in the list. The third argument is optional, if you use :prune then the Map returned will only contain the keys that are in the key_map"},{"ref":"KitchenSink.Map.html#remapper/3","title":"KitchenSink.Map.remapper/3","type":"function","doc":"Takes a list of tuples of path(s) that describe from-&gt;to remappings of an object one can consider this to be like remap_keys or Map.take, but for nested keys. for each tuple of paths, the value of the first path will move to the second path. if there is only one path provided, then this will act more like Map.take Example iex&gt; KitchenSink.Map.remapper(%{a: %{b: 1, c: 2}, d: 3}, [{[:a, :c]}]) %{a: %{c: 2}} iex&gt; KitchenSink.Map.remapper(%{a: %{b: 1, c: 2}, d: 3}, [{[:a, :c], [:d, :z]}]) %{d: %{z: 2}} iex&gt; KitchenSink.Map.remapper(%{a: %{b: 1, c: 2}, d: 3}, [{[:e], [:d, :c]}]) %{} This function prunes the input map"},{"ref":"KitchenSink.Map.html#rename_key/2","title":"KitchenSink.Map.rename_key/2","type":"function","doc":"rename_key remaps a value from one key to another in a map Example: iex&gt; import KitchenSink.Map iex&gt; rename_key %{a: 1, b: 2, c: 3, d: 4}, :a, :z %{z: 1, b: 2, c: 3, d: 4}"},{"ref":"KitchenSink.Map.html#rename_key/4","title":"KitchenSink.Map.rename_key/4","type":"function","doc":""},{"ref":"KitchenSink.Map.html#transform/1","title":"KitchenSink.Map.transform/1","type":"function","doc":"like transform/2, but returns a function that takes a Map to be transformed. Examples iex&gt; KitchenSink.Map.transform(%{b: {[:b1, :b2], &amp;Float.round(&amp;1, 2)}}).(%{a: 1, b: 0.9876}) %{a: 1, b1: %{b2: 0.99}} output is a function that takes a Map. the output Map is made from appling each of the transformers in the transformer-map to the corresponding keys and values in the Map, outputing a Map where each key-value in the Map has been transformed."},{"ref":"KitchenSink.Map.html#transform/2","title":"KitchenSink.Map.transform/2","type":"function","doc":"like transform/3 but doesn&#39;t prune the output Map, preserving key value pairs that are not part of the transformation_map Examples iex&gt; KitchenSink.Map.transform(%{a: 1, b: 0.9876}, %{b: {[:b1, :b2], &amp;Float.round(&amp;1, 2)}}) %{a: 1, b1: %{b2: 0.99}} output is a transformed Map. the output Map is made from appling each of the transformers in the transformer-map to the corresponding keys and values in the Map, outputing a Map where each key-value in the Map has been transformed. supplying prune: true prunes the map so only transformed values are output."},{"ref":"KitchenSink.Map.html#transform/3","title":"KitchenSink.Map.transform/3","type":"function","doc":"transforms the keys and values of a map based on a map or tuple {key_list, function}. input is a map of keys to tuples {key_list, function}, representing a transformer-map. key_list = any || [any, ...] function = (any -&gt; any) Examples iex&gt; KitchenSink.Map.transform(%{b: 0.9876}, %{b: {[:b1, :b2], &amp;Float.round(&amp;1, 2)}}, prune: true) %{b1: %{b2: 0.99}} output is a transformed Map. the output Map is made from appling each of the transformers in the transformer-map to the corresponding keys and values in the Map, outputing a Map where each key-value in the Map has been transformed. supplying prune: true prunes the map so only transformed values are output."},{"ref":"KitchenSink.Map.html#transform_values/1","title":"KitchenSink.Map.transform_values/1","type":"function","doc":"transforms the values of a map based on a map of functions. input is a map of keys to functions, representing a transformer-map. %{ a: a_transform_fun, b: b_transform_fun, ... } output is a function that takes a map and applies each of the transformers in the transformer-map to the corresponding value in the map, outputing a map where each key-value in the map has been transformed. prunes the map so only transformed values are output. fn(%{a:, b: ...}) -&gt; %{a: a_transform_fun(a), b: b_transform_fun(b) ...}"},{"ref":"KitchenSink.Map.html#transform_values/2","title":"KitchenSink.Map.transform_values/2","type":"function","doc":"transforms the values of a map based on a map of functions. similar to transform_values/1, however doesn&#39;t return a function."},{"ref":"KitchenSink.Map.html#trim/1","title":"KitchenSink.Map.trim/1","type":"function","doc":"Walks an object and recursively removes nodes with nil/empty values Works for Structs as well!"},{"ref":"KitchenSink.Map.html#trim/2","title":"KitchenSink.Map.trim/2","type":"function","doc":""},{"ref":"KitchenSink.Math.html","title":"KitchenSink.Math","type":"module","doc":"Math functions!"},{"ref":"KitchenSink.Math.html#ceil/2","title":"KitchenSink.Math.ceil/2","type":"function","doc":"See KitchenSink.Math.round_up_to_multiple/2."},{"ref":"KitchenSink.Math.html#div/3","title":"KitchenSink.Math.div/3","type":"function","doc":"Numerical division which allows a specified return for &quot;divide by zero.&quot; iex&gt; div(1, 2, :oops) 0.5 iex&gt; div(1, 0, :oops) :oops"},{"ref":"KitchenSink.Math.html#floor/2","title":"KitchenSink.Math.floor/2","type":"function","doc":"See KitchenSink.Math.round_down_to_multiple/2."},{"ref":"KitchenSink.Math.html#round_down_to_multiple/2","title":"KitchenSink.Math.round_down_to_multiple/2","type":"function","doc":"Round down to the nearest multiple of significance. Parameters number: The number you wish to have rounded. significance: The multiple to which you would like to round. Must not be 0. Examples: iex&gt; round_down_to_multiple(31, 5) 30 iex&gt; round_down_to_multiple(29.49, 10) 20"},{"ref":"KitchenSink.Math.html#round_up_to_multiple/2","title":"KitchenSink.Math.round_up_to_multiple/2","type":"function","doc":"Always round up to the nearest multiple of significance. Parameters number: The number you wish to have rounded. significance: The multiple to which you would like to round. Must not be 0. Examples: iex&gt; round_up_to_multiple(31, 5) 35 iex&gt; round_up_to_multiple(29.49, 10) 30"},{"ref":"KitchenSink.Math.html#t:error_return/0","title":"KitchenSink.Math.error_return/0","type":"type","doc":"Some functions allow us to specify something to return in a case which would normally raise an error. This type identifies that."},{"ref":"KitchenSink.Matrix.html","title":"KitchenSink.Matrix","type":"module","doc":"this module is for dealing with matrix (2d array) operations"},{"ref":"KitchenSink.Matrix.html#transpose/1","title":"KitchenSink.Matrix.transpose/1","type":"function","doc":"This will take a 2D array and map all of it&#39;s indicies [x,y] -&gt; [y,x]"},{"ref":"KitchenSink.Struct.html","title":"KitchenSink.Struct","type":"module","doc":"functions that apply to structs"},{"ref":"KitchenSink.Struct.html#to_map/2","title":"KitchenSink.Struct.to_map/2","type":"function","doc":"convert a struct to a map removing all structness"},{"ref":"KitchenSink.StructAccess.html","title":"KitchenSink.StructAccess","type":"module","doc":"This module makes structs use Access to get and change their values. To use it for your struct do use KitchenSink.StructAccess"},{"ref":"KitchenSink.StructEnum.html","title":"KitchenSink.StructEnum","type":"module","doc":"This module makes structs Enumerable which means that you can use Enum functions on them. The format of default values we are using is setting them to nil so count uses that. To use it for your struct do use KitchenSink.StructEnum"},{"ref":"readme.html","title":"Kitchen Sink","type":"extras","doc":"Kitchen Sink Module to make improvements to the Elixir Standard API. This module should follow Elixir guidelines, and we should consider everything in these files as able to be patched into mainstream Elixir. This is a good place to adopt APIs from other languages or frameworks (Ramda, Clojure, Elm, Haskell, etc...)"},{"ref":"readme.html#installation","title":"Kitchen Sink - Installation","type":"extras","doc":"If available in Hex, the package can be installed as: Add kitchen_sink to your list of dependencies in mix.exs: def deps do [{:kitchen_sink, &quot;~&gt; 1.2.0&quot;}] end Ensure kitchen_sink is started before your application: def application do [applications: [:kitchen_sink]] end"},{"ref":"changelog.html","title":"KitchenSink","type":"extras","doc":"KitchenSink"},{"ref":"changelog.html#changes-since-1-3-1","title":"KitchenSink - Changes since 1.3.1","type":"extras","doc":"Refactor and test the binary search algorithm"},{"ref":"changelog.html#changes-since-1-3-0","title":"KitchenSink - Changes since 1.3.0","type":"extras","doc":"Added KitchenSink.Integer ensure_integer/1 Add drop option to Struct.to_map"},{"ref":"changelog.html#changes-since-1-2-0","title":"KitchenSink - Changes since 1.2.0","type":"extras","doc":"Bump Elixir version to 1.4.5 Update dialyxir - dev only Use asdf&#39;s .tool-versions to define versions of Erlang and Elixir"},{"ref":"changelog.html#changes-since-1-1-0","title":"KitchenSink - Changes since 1.1.0","type":"extras","doc":"Added Math.floor/2 delegated to Math.round_down_to_multiple/2 Add @olafura to maintainers list Fix Struct.to_map/1 for list of primitives"},{"ref":"changelog.html#changes-since-1-0-2","title":"KitchenSink - Changes since 1.0.2","type":"extras","doc":"Added CHANGELOG.md Added KitchenSink.Algorithms binary_search/{2,3,4} Updated credo 0.6.1 → 0.8.1 Updated dialyxir 0.4.4 → 0.5.0 Updated ex_doc 0.14.5 → 0.16.1"}]